%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%5
% TAR_Rate.M
% To estimate and test a threshold bi-variate VECM

% written by:

% Bruce E. Hansen
% Department of Economics
% Social Science Building
% University of Wisconsin
% Madison, WI 53706-1393
% bhansen@ssc.wisc.edu
% http://www.ssc.wisc.edu/~bhansen/

% and 

% Byeongseon Seo
% Department of Economics
% Soongsil University
% Seoul, 156-743
% Korea
% seo@saint.soongsil.ac.kr


% This Matlab program estimates a bi-variate VECM, a threshold bi-variate VECM, 
% and tests for the presence of a threshold.  The methods are those described in
% "Testing for Threshold Cointegration" by Bruce E. Hansen and Byeongseon Seo.

% The program is set up to replicate the empirical application to the 3-month and 6-month
% interest rates series.  For your own application, load your data into the matrix "dat", and change the
% controls listed below


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%  Controls %
function tar_rate();
global x;
global y;
global t;
global xlag;
global gn;
global k;
global ri;
global trim_;
global graph_rotate_;
gn=300;			% number of gridpoints for gamma @
bn = 300;		% number of gridpoints for beta      @
trim_=.05;		% trimming percentage for threshold @
boot=5000;		% number of bootstrap replications 
		        % set equal to zero to not do testing  @
coint_=0;		% set to 1 to estimate cointegrating vector
			    % set to 0 to fix cointegrating vector at _cvalue @
cvalue_=1;		% cointegrating vector, if coint_=0 @
cov_=1;		    % covariance matrix estimation method
		        % set to 1 for Eicker-White
		        % set to 0 for conventional homoskedastic estimator @
p_ests_=1;		% set to 1 to print estimates, else 0 @
graph_=1;		% set to 1 to generate graph of nonlinear ECM, else 0 @
graph_rotate_=0;	% set to 1 to generate rotated graphs 
		        %(useful for some print jobs, but ackward for screen viewing) @
  

% Load your own data  into matrix "dat" @
% Here we load in the 3-month and 6-month T-Bill series @

load zeroyld.txt;
dat=zeroyld;
datstore=dat(:,7:62);
rs=[(0:18)';21;24;30;(36:12:120)'];

rates_i=[1 2;
1 3;
1 6;
3 6;
3 12;
3 120;
12 24;
12 120;
24 120];
out=fopen('tar_rate.txt','wt');
fclose(out);
k=1;			% Lags in VAR beyond EC @
while k<=2
    ri=1;
    while ri<=length(rates_i(:,1))
        short=rates_i(ri,1);
        long=rates_i(ri,2);
        [temp,short_i]=max(rs==short);
        [temp,long_i]=max(rs==long);
        dat=datstore(:,[long_i,short_i]);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Main Program
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

        if p_ests_==1;
            out=fopen('tar_rate.txt','at');
            fprintf(out,'**************************************************\n');
            fprintf(out,'Number of Bootstrap Replications:             %u\n',boot);
            fprintf(out,'Number of Gridpoints for threshold:           %u\n',gn);
            if coint_==1
                fprintf(out,'Estimated Cointegrating Vector\n');
                fprintf(out,'Number of Gridpoints for ci vector:           %u\n',bn);
            else
                fprintf(out,'Cointegrating Vector fixed at:                %f\n',cvalue_);
            end;
            fprintf(out,'\n\n');
            fprintf(out,'Long Rate  (month):                               %u\n',long);
            fprintf(out,'Short Rate (month):                               %u\n',short);
            fprintf(out,'Number of VAR lags:                               %u\n',k);
            fprintf(out,'\n');
            fclose(out);
        end;

        % Organize Data %
        n=length(dat(:,1));
        y=dat(2+k:n,:)-dat(1+k:n-1,:);
        t=length(y(:,1));
        xlag=dat(1+k:n-1,:);
        x=ones(t,1);
        j=1;
        while j<=k
            x=[x,dat(2+k-j:n-j,:)-dat(1+k-j:n-1-j,:)];
            j=j+1;
        end;

        % Compute Linear Model %
        xx=inv(x'*x);
        xxx=x*xx;
        u=y-x*xx*(x'*y);
        if coint_==1
            v=xlag-x*xx*(x'*xlag);
            uu=u'*u;
            m=inv(v'*v)*(v'*u)*inv(uu)*(u'*v);
            [ve,va]=eig(m);
            va=eig(m);
            [temp,maa]=max(va);
            h=ve(:,maa);
            b0=-h(2)/h(1);
        else
            b0=cvalue_;
        end;
        w0=xlag*[1;-b0];
        z0=[w0,x];
        kk=length(z0(1,:));
        zz0=inv(z0'*z0);
        zzz0=z0*zz0;
        beta0=zzz0'*y;
        e=y-z0*beta0;
        sige=e'*e/t;
        nlike=(t/2)*log(det(sige));
        bic=nlike+log10(t)*4*(1+k);
        aic=nlike+2*4*(1+k);
        nlike1=b_like(b0+.001);
        nlike2=b_like(b0-.001);
        hp=(nlike1+nlike2-2*nlike)/(.001^2);
        seb=1/sqrt(hp);
        if cov_==1
            xe=[(z0.*(e(:,1)*ones(1,length(z0(1,:))))),(z0.*(e(:,1)*ones(1,length(z0(1,:)))))];
            m0=[zz0,zeros(length(zz0(:,1)),length(zz0(1,:)));zeros(length(zz0(:,1)),length(zz0(1,:))),zz0];
            v=m0*(xe'*xe)*m0;
        else
            v=sige.*zz0;
        end;
        se=sqrt(diag(v));

        if p_ests_==1;
            out=fopen('tar_rate.txt','at');
            fprintf(out,'Linear VECM Estimates\n');
            fprintf(out,'\n');
            if coint_==1;
                fprintf(out,'Cointegrating Vector:  %f   %f\n',b0,seb);
            else
                fprintf(out,'Cointegrating Vector:  %f\n',b0);
            end;
            fprintf(out,'Negative Log-Like:         %f\n',nlike);
            fprintf(out,'AIC:                       %f\n',aic);
            fprintf(out,'BIC:                       %f\n',bic);
            fprintf(out,'\n');
            fprintf(out,'Equation 1\n');
            for i=1:kk
                fprintf(out,'%f   %f\n',beta0(i,1),se(i));
            end;
            fprintf(out,'\n');
            for i=kk+1:2*kk
                fprintf(out,'%f   %f\n',beta0(i-kk,2),se(i));
            end;
            fprintf(out,'\n\n\n');
            fclose(out);
        end;

        % Set-up Grids %
        q=unique(w0);
        gamma1=q(round((trim_:(1-2*trim_)/gn:(gn-1)*((1-2*trim_)/gn)+trim_)*length(q(:,1))));
        gamma2=q(round((1/(gn+1):1/(gn+1):(gn/(gn+1)))*length(q(:,1))));
        if coint_==1
            if bn==1
                beta==b0;
            else
                betas=(b0-6*seb:12*seb/(bn-1):b0-6*seb+12*seb);
            end;
        else
            betas=b0;
            bn=1;
        end;
        % Estimate Threshold Model %
        store=zeros(gn,bn)+nlike;
        j=1;
        while j<=gn
            gam=gamma2(j);
            bj=1;
            while bj<=bn
                betab=betas(bj);
                w=xlag*[1;-betab];
                z=[w,x];
                d1=(w<=gam);
                n1=sum(d1)';
                if min([n1;t-n1])/t>trim_
                    zj=[(z.*(d1*ones(1,length(z(1,:))))),w];
                    zzj=zj-x*xx*(x'*zj);
                    warning off;
                    lastwarn(' ');
                    bz=(u'/zzj')';
                    [mw,idw] = lastwarn;
                    lastwarn(' ');
                    warning on;
                    if (1-mw==' ');
                        bz=pinv(zzj'*zzj)*(zzj'*u);
                    end;
                    store(j,bj)=(t/2)*log(det((u-zzj*bz)'*(u-zzj*bz)/t));
                end;
                bj=bj+1;
            end;
            j=j+1;
        end;
        [temp,m]=min(store);
        [temp,c]=min(diag(store(m,:)));
        r=m(c);
        gammahat=gamma2(r);
        b1=betas(c);
        nlike=store(r,c);
        bic=nlike+log10(t)*8*(1+k);
        aic=nlike+2*8*(1+k);
        w=xlag*[1;-b1];
        z=[w,x];
        d1=(w<=gammahat);
        d2=1-d1;
        ind=0;
        for i=1:length(d1)
            if d1(i)==1
                if ind==0
                    y1=y(i,:);
                    z1=z(i,:);
                    ind=1;
                else
                    y1=[y1;y(i,:)];
                    z1=[z1;z(i,:)];
                end;
            end;
        end;
        ind=0;
        for i=1:length(d2);
            if d2(i)==1
                if ind==0
                    y2=y(i,:);
                    z2=z(i,:);
                    ind=1;
                else
                    y2=[y2;y(i,:)];
                    z2=[z2;z(i,:)];
                end;
            end;
        end;
        zz1=inv(z1'*z1);
        zz2=inv(z2'*z2);
        beta1=zz1*z1'*y1;
        beta2=zz2*z2'*y2;
        e1=y1-z1*beta1;
        e2=y2-z2*beta2;
        if cov_==1
            xe1=[(z1.*(e1(:,1)*ones(1,length(z1(1,:))))),(z1.*(e1(:,2)*ones(1,length(z1(1,:)))))];
            m1=[zz1,zeros(length(zz1(:,1)),length(zz1(1,:)));zeros(length(zz1(:,1)),length(zz1(1,:))),zz1];
            v1=m1*xe1'*xe1*m1;
            xe2=[(z2.*(e2(:,1)*ones(1,length(z2(1,:))))),(z2.*(e2(:,2)*ones(1,length(z2(1,:)))))];
            m2=[zz2,zeros(length(zz2(:,1)),length(zz2(1,:)));zeros(length(zz2(:,1)),length(zz2(1,:))),zz2];
            v2=m2*xe2'*xe2*m2;
        else
            sig1=e1'*e1/length(e1(:,1));
            indx=0;
            indy=0;
            for ii=1:length(sig1(:,1))
                for jj=1:length(sig1(1,:))
                    if indx==0
                        tempx=zz1*sig1(ii,jj);
                        indx=1;
                    else
                        tempx=[tempx,zz1*sig1(ii,jj)];
                    end;
                end;
                indx=0;
                if indy==0
                    v1=tempx;
                    indy=1;
                else
                    v1=[v1;tempx];
                end;
            end;           
            sig2=e2'*e2/length(e2(:,1));
            indx=0;
            indy=0;
            for ii=1:length(sig2(:,1))
                for jj=1:length(sig2(1,:))
                    if indx==0
                        tempx=zz2*sig2(ii,jj);
                        indx=1;
                    else
                        tempx=[tempx,zz2*sig2(ii,jj)];
                    end;
                end;
                indx=0;
                if indy==0
                    v2=tempx;
                    indy=1;
                else
                    v2=[v2;tempx];
                end;
            end;
        end;
        se1=sqrt(diag(v1));
        se2=sqrt(diag(v2));
        rb=length(beta1);
        bb=beta1-beta2;
        if k>0
            ind=0;% vec %
            for jj=1:length(bb(1,:))
                for ii=3:rb
                    if ind==0
                        bw=bb(ii,jj);
                        ind=1;
                    else
                        bw=[bw;bb(ii,jj)];
                    end;
                end;
            end;
            vr=[(3:rb)';(rb+3:2*rb)'];
            vv=v1(vr,vr)+v2(vr,vr);
            ww=bw'*inv(vv)*bw;
        end;
        wecm=bb(1,:)*inv(v1([1,rb+1],[1,rb+1])+v2([1,rb+1],[1,rb+1]))*(bb(1,:)');

        if p_ests_==1
            out=fopen('tar_rate.txt','at');
            fprintf(out,'Threshold VECM Estimates\n\n');
            fprintf(out,'Threshold Estimate:                    %f\n',gammahat);
            fprintf(out,'Cointegrating Vector Estimate:         ');
            for i=1:length(b1)
                fprintf(out,'%f   ',b1(i));
            end;
            fprintf(out,'\n');
            fprintf(out,'Negative Log-Like:                     %f\n',nlike);
            fprintf(out,'AIC:                                   %f\n',aic);
            fprintf(out,'BIC:                                   %f\n',bic);
            fprintf(out,'\n');
            fprintf(out,'First Regime\n');
            fprintf(out,'Percentage of Obs:                     %f\n',mean(d1));
            fprintf(out,'\n');
            fprintf(out,'  Equation 1\n');
            for i=1:kk
                fprintf(out,'%f   %f\n',beta1(i,1),se1(i));
            end;
            fprintf(out,'\n');
            fprintf(out,'  Equation 2\n');
            for i=kk+1:2*kk
                fprintf(out,'%f   %f\n',beta1(i-kk,2),se1(i));
            end;
            fprintf(out,'\n');
            fprintf(out,'Second Regime\n');
            fprintf(out,'Percentage of Obs:                     %f\n',mean(d2));
            fprintf(out,'\n');
            fprintf(out,'  Equation 1\n');
            for i=1:kk
                fprintf(out,'%f   %f\n',beta2(i,1),se2(i));
            end;
            fprintf(out,'\n');
            fprintf(out,'  Equation 2\n');
            for i=kk+1:2*kk
                fprintf(out,'%f   %f\n',beta2(i-kk,2),se2(i));
            end;
            fprintf(out,'\n');
            if k>0;
                fprintf(out,'Wald Test for Equality of Dynamic Coefs: %f   %f\n',ww,1-chi2cdf(ww,(rb-2)*2));
            end;
            fprintf(out,'Wald Test for Equality of ECM Coef:      %f   %f\n',wecm,1-chi2cdf(wecm,2));
            fprintf(out,'\n\n');
            fclose(out);
        end;

        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        if graph_==1|graph_rotate_==1
            if coint_==1
                loglike=min(store)';
                plot(betas,loglike);
                title('Figure 2. LConcentrated Negative Log Likelihood');
                ylabel('Negative Log-Likelihood');
                xlabel('Beta');
                grid on;
                saveas(gcf,strcat('tar_ratefig2_',num2str(k),num2str(ri),'.eps'),'psc2');
            end;
            if graph_rotate_==1
                title('Figure 2. Concentrated Negative Log Likelihood');
                protate=1;
                plot(loglike,betas);
                xlabel('Negative Log-Likelihood');
                ylabel('Beta');
                saveas(gcf,strcat('tar_ratefig2_',num2str(k),num2str(ri),'.eps'),'psc2');
            end;
        end;
        title('Figure 1. Concentrated Negative Log Likelihood');
        loglike=min(store')';
        if length(store(1,:))==1
            loglike=store;
        end;
        plot(gamma2,loglike);
        ylabel('Negative Log-Likelihood');
        xlabel('Gama');
        grid on;
        saveas(gcf,strcat('tar_ratefig1_',num2str(k),num2str(ri),'.eps'),'psc2');
        if graph_rotate_==1
            plot(loglike,gamma2); 
            xlabel('Negative Log-Likelihood');
            ylabel('Gama');
            saveas(gcf,strcat('tar_ratefig1_',num2str(k),num2str(ri),'.eps'),'psc2');
        end;
        grid on;
        zbeta1=z(:,[1 2])*beta1([1 2],:);
        zbeta2=z(:,[1 2])*beta2([1 2],:);
        yhat2=zbeta1.*(d1*ones(1,length(zbeta1(1,:))))+(z(:,[1 2])*beta2([1 2],:)).*(d2*ones(1,length(zbeta2(1,:))));
        wi=0; % Sortind %
        for i=1:length(w)
            for i_=1:length(w)
                temp=0;
                for j=1:length(w)
                    if (w(i_)>w(j))|(i_>=j&w(i_)==w(j))
                        temp=temp+1;
                    end;
                end;
                if temp==i
                    wi=[wi;i_];
            end;
            end;
        end;
        wi=wi(2:length(wi));

        yhat2=yhat2(wi,:);
        wi=w(wi);


        plot(wi,yhat2(:,1),wi,yhat2(:,2));
        title('Interest Rate Response to Error-Correction');
        ylabel('Interest Rate Response');
        xlabel(strcat('Error Correction: R(',num2str(long),')(t-1)-Beta*R(',num2str(short),')(t-1)'));
        legend(strcat('Long Rate R(',num2str(long),')'),strcat('Short Rate R(',num2str(short),')'));
        grid on;
        saveas(gcf,strcat('tar_ratefig3_',num2str(k),num2str(ri),'.eps'),'psc2');
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%5555
        % Testing %
        lm01=lmtest01(y,x,w0,gamma1,graph_);
        if p_ests_==1
            out=fopen('tar_rate.txt','at');
            fprintf(out,'Lagrange Multipler Threshold Test\n\n Test Statistic:     %f\n',lm01);
            fclose(out);
        end;

        if boot>0
            % Fixed Regressor Bootstrap %
            fix01=zeros(boot,1);
            r=1;
            while r<=boot
                yr=normrnd(0,1,t,2).*e;
            fix01(r)=lmtest01(yr,x,w0,gamma1,0);
            r=r+1;
            end;
            pfix01=mean(fix01>lm01)';
            fix01=sortrows(fix01,1);
            cfix01=fix01(round(.95*boot),:);
    
            % Parametric Bootstrap %
            x0=dat(1:1+k,:);
            mu=beta0(2,:);
            ab=[1;-b0]*beta0(1,:);
            boot01=zeros(boot,1);
            if k>0
                capgamma=beta0(3:2+2*k,:);
                temp=x0(k+1,:)-x0(k,:);
                for ii=k:(-1):2
                    temp=[temp;x0(ii,:)-x0(ii-1,:)];
                end;
                ind=0;
                for ii=1:length(temp(:,1))
                    for jj=1:length(temp(1,:))
                        if ind==0
                            dx0=temp(ii,jj);
                            ind=1;
                        else
                            dx0=[dx0;temp(ii,jj)];
                        end;
                    end;
                end;
            end;
            r=1;
            while r<=boot
                datb=x0;
                if k>0
                    dx=dx0;
                end;
                datbi=datb(k+1,:);
                ei=ceil(unifrnd(0,1,t,1)*t);
                eb=e(ei,:);
                i=k+2;
                while i<=n
                    u=mu+eb(i-k-1,:)+datbi*ab;
                    if k>0
                        u=u+dx'*capgamma;
                    end;
                    datbi=datbi+u;
                    datb=[datb;datbi];
                    if k==1
                        dx=u';
                    elseif k>1
                        dx=[u';dx(1:2*k-2)];
                    end;
                    i=i+1;
                end;
                yr=datb(2+k:n,:)-datb(1+k:n-1,:);
                xlagr=datb(1+k:n-1,:);
                xr=ones(t,1);
                j=1;
                while j<=k
                    xr=[xr,(datb(2+k-j:n-j,:)-datb(1+k-j:n-1-j,:))];
                    j=j+1;
                end;
                if coint_==1
                    xxr=inv(xr'*xr);
                    u=yr-xr*xxr*(xr'*xlagr);
                    v=xlagr-xr*xxr*(xr'*xlagr);
                    m=inv(v'*v)*(v'*u)*inv(u'*u)*(u'*v);
                    [ve,va]=eig(m);
                    va=eig(m);
                    [temp,maxindva]=max(va);
                    h=ve(:,maxindva);
                    wr=xlagr*[1;(h(2)/h(1))];
                else
                    wr=xlagr*[1;-cvalue_];
                end;
                boot01(r)=lmtest01(yr,xr,wr,gamma1,0);
                r=r+1;
            end;
            boot01=sortrows(boot01,1);
            cb01=boot01(round(.95*boot));
            pb01=mean(boot01>lm01);
    
            if p_ests_==1;
                fopen('tar_rate.txt','at');
                fprintf(out,'Fixed Regressor (Asymptotic) .05 Critical Value:    %f\n',cfix01);
                fprintf(out,'Bootstrap .05 Critical Value:                       %f\n',cb01);
                fprintf(out,'Fixed Regressor (Asymptotic) P-Value:              %f\n',pfix01);
                fprintf(out,'Bootstrap P-Value:                                 %f\n',pb01);
                fprintf(out,'\n\n');
                fclose(out);
            end;
        end;
        ri=ri+1;
    end;
    k=k+1;
end;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function like=b_like(b);
global x;
global y;
global t;
global xlag;
z=[xlag*[1;-b],x];
sigma=((y-z*(y'/z')')'*(y-z*(y'/z')'))/t;
like=(t/2)*log(det(sigma));

% Proc for LM test %
function lm01=lmtest01(y,x,w0,gammas,plot_);
global graph_rotate_;
global gn;
global t;
global trim_;
global k;
global ri;
z0=[w0,x];
warning off;
lastwarn(' ');
z0zz=z0*inv(z0'*z0);
[mw,idw] = lastwarn;
lastwarn(' ');
warning on;
if (1-mw==' ')
    z0zz=z0*pinv(z0'*z0);
end;
e=y-z0zz*(z0'*y);
e1=e(:,1);
e2=e(:,2);
store=zeros(gn,1);
j=1;
while j<=gn
    d1=w0<=gammas(j);
    n1=sum(d1);
    if min([n1;(t-n1)])/t>trim_
        z1=z0.*(d1*ones(1,length(z0(1,:))));
        z11=z1-z0zz*(z0'*z1);
        ze=[(z11.*(e1*ones(1,length(z11(1,:))))),(z11.*(e2*ones(1,length(z11(1,:)))))];
        v=ze'*ze;
        z11y=z11'*y;
        ind=0;% vec %
        for jj=1:length(z11y(1,:))
            for ii=1:length(z11y(:,1))
                if ind==0
                    s=z11y(ii,jj);
                    ind=1;
                else
                    s=[s;z11y(ii,jj)];
                end;
            end;
        end;
        warning off;
        lastwarn(' ');
        sv=(s'/v')';
        [mw,idw] = lastwarn;
        lastwarn(' ');
        warning on;
        if (1-mw==' ')
             sv=pinv(v)*s;
        end;
        store(j)=s'*sv;
    end;
    j=j+1;
end;
lm01=max(store);
if plot_==1
    plot(gammas,store);
    xlabel('Gamma'); 
    grid on;
    title('LM Statistic as function of Gamma');
    saveas(gcf,strcat('tar_ratefig4_',num2str(k),num2str(ri),'.eps'),'psc2');
    if graph_rotate_==1
        plot(store,gammas);
        xlabel('');
        ylabel('Gamma');
        saveas(gcf,strcat('tar_ratefig4_',num2str(k),num2str(ri),'.eps'),'psc2');
    end;
end;
              
        
    




    
    
    
    
    
    
            
            
            

    